'use strict';

var _require = require('hapi');

var Server = _require.Server;

var Queue = require('bee-queue');
var axios = require('axios');

var queueMap = {};

/**
 * The server entry point.
 * @param {number} port The port number
 */
var main = function main(port) {

  var server = new Server();

  server.connection({ port: port });

  server.route({
    method: 'POST',
    path: '/jobs/{name}/payloads',
    handler: function handler(request, reply) {
      var payloads = request.payload;

      if (!Array.isArray(payloads)) {
        payloads = [payloads];
      }

      var name = request.params.name;
      var queue = queueMap[name];

      if (!queue) {
        return reply({ code: 'NO_SUCH_JOB' });
      }

      payloads.forEach(function (payload) {
        return queue.createJob(payload).save();
      });

      reply({ ok: true });
    }
  });

  server.route({
    method: 'POST',
    path: '/jobs/{name}',
    handler: function handler(request, reply) {
      var job = request.payload;

      var name = request.params.name;

      if (queueMap[name]) {
        return reply({
          code: 'JOB_NAME_UNAVALABLE',
          description: 'There is already an job with the same name: ' + name + '.'
        });
      }

      if (job == null) {
        return reply({
          code: 'INVALID_JOB',
          description: 'A job should be an object. null is given.'
        });
      }

      if (!/GET|POST|PUT|DELETE|HEAD/.test(job.method)) {
        return reply({
          code: 'INVALID_JOB',
          description: 'The job method is invalid: ' + job.method
        });
      }

      var method = job.method;
      var url = job.url;

      var queue = queueMap[name] = new Queue(name);

      console.log('Creating job: ' + name);

      queue.process(function (job, done) {
        var data = job.data;
        console.log('processing ' + name + ' ' + data);
        axios({ method: method, url: url, data: data }).then(function (data) {
          return done(url, data);
        }).catch(function (err) {
          console.log(err.stack);done(err);
        });
      });

      reply({
        ok: true,
        description: 'A job is created. The name is \'' + name + '\'.',
        job: { name: name, method: method, url: url }
      });
    }
  });

  server.start(function (err) {
    if (err) {
      throw err;
    }

    console.log('Server running at:', server.info.uri);
  });
};

module.exports = main;